@@ -1,13 +1,13 @@
--- | Simple, high-level DNS lookup functions.
+-- | Simple, high-level DNS lookup functions for clients.
 --
---   All of the lookup functions necessary run in IO, since they
+--   All of the lookup functions necessary run in IO since they
 --   interact with the network. The return types are similar, but
 --   differ in what can be returned from a successful lookup.
 --
---   We can think of the return type as \"either what I asked for, or
---   an error\". For example, the 'lookupA' function, if successful,
+--   We can think of the return type as either \"what I asked for\" or
+--   \"an error\". For example, the 'lookupA' function, if successful,
 --   will return a list of 'IPv4'. The 'lookupMX' function will
---   instead return a list of @('Domain',Int)@ pairs, where each pair
+--   instead return a list of @('Domain','Int')@ pairs, where each pair
 --   represents a hostname and its associated priority.
 --
 --   The order of multiple results may not be consistent between
@@ -42,57 +42,60 @@
 --   one millisecond and a very limited number of retries:
 --
 --   >>> let hostname = Data.ByteString.Char8.pack "www.example.com"
---   >>> let badrc = defaultResolvConf { resolvTimeout = 1, resolvRetry = 1 }
+--   >>> let badrc = defaultResolvConf { resolvTimeout = 0, resolvRetry = 1 }
 --   >>>
 --   >>> rs <- makeResolvSeed badrc
 --   >>> withResolver rs $ \resolver -> lookupA resolver hostname
 --   Left RetryLimitExceeded
 --
 --   As is the convention, successful results will always be wrapped
---   in a 'Right', while errors will be wrapped in a 'Left'.
+--   in a 'Right' while errors will be wrapped in a 'Left'.
 --
---   For convenience, you may wish to enable GHC's OverloadedStrings
+--   For convenience, you may wish to enable GHC\'s OverloadedStrings
 --   extension. This will allow you to avoid calling
 --   'Data.ByteString.Char8.pack' on each domain name. See
---   <http://www.haskell.org/ghc/docs/7.6.3/html/users_guide/type-class-extensions.html#overloaded-strings>
---   for more information.
+--   <https://downloads.haskell.org/~ghc/latest/docs/html/users_guide/glasgow_exts.html#overloaded-string-literals>
+--   for more information. In the following examples,
+--   we assuem this extension is enabled.
 --
+--   All lookup functions eventually call 'lookupRaw'. See its manual
+--   to understand the concrete lookup behavior.
 module Network.DNS.Lookup (
     lookupA, lookupAAAA
   , lookupMX, lookupAviaMX, lookupAAAAviaMX
   , lookupNS
   , lookupNSAuth
   , lookupTXT
+  , lookupSOA
   , lookupPTR
   , lookupRDNS
   , lookupSRV
   ) where
 
-import Data.ByteString (ByteString)
 import qualified Data.ByteString.Char8 as BS
 import Data.IP (IPv4, IPv6)
+
+import Network.DNS.Imports
+import Network.DNS.LookupRaw as DNS
 import Network.DNS.Resolver as DNS
 import Network.DNS.Types
 
+
 ----------------------------------------------------------------
 
 -- | Look up all \'A\' records for the given hostname.
 --
 --   A straightforward example:
 --
---   >>> let hostname = Data.ByteString.Char8.pack "www.mew.org"
---   >>>
 --   >>> rs <- makeResolvSeed defaultResolvConf
---   >>> withResolver rs $ \resolver -> lookupA resolver hostname
+--   >>> withResolver rs $ \resolver -> lookupA resolver "www.mew.org"
 --   Right [210.130.207.72]
 --
 --   This function will also follow a CNAME and resolve its target if
 --   one exists for the queries hostname:
 --
---   >>> let hostname = Data.ByteString.Char8.pack "www.kame.net"
---   >>>
 --   >>> rs <- makeResolvSeed defaultResolvConf
---   >>> withResolver rs $ \resolver -> lookupA resolver hostname
+--   >>> withResolver rs $ \resolver -> lookupA resolver "www.kame.net"
 --   Right [203.178.141.194]
 --
 lookupA :: Resolver -> Domain -> IO (Either DNSError [IPv4])
@@ -112,10 +115,8 @@ lookupA rlv dom = do
 --
 --   Examples:
 --
---   >>> let hostname = Data.ByteString.Char8.pack "www.wide.ad.jp"
---   >>>
 --   >>> rs <- makeResolvSeed defaultResolvConf
---   >>> withResolver rs $ \resolver -> lookupAAAA resolver hostname
+--   >>> withResolver rs $ \resolver -> lookupAAAA resolver "www.wide.ad.jp"
 --   Right [2001:200:dff:fff1:216:3eff:fe4b:651c]
 --
 lookupAAAA :: Resolver -> Domain -> IO (Either DNSError [IPv6])
@@ -141,18 +142,14 @@ lookupAAAA rlv dom = do
 --   examples posted on the internet). But remember, \"no results\" is
 --   still a successful result.
 --
---   >>> let hostname = Data.ByteString.Char8.pack "example.com"
---   >>>
 --   >>> rs <- makeResolvSeed defaultResolvConf
---   >>> withResolver rs $ \resolver -> lookupMX resolver hostname
+--   >>> withResolver rs $ \resolver -> lookupMX resolver "example.com"
 --   Right []
 --
 --   The domain \"mew.org\" does however have a single MX:
 --
---   >>> let hostname = Data.ByteString.Char8.pack "mew.org"
---   >>>
 --   >>> rs <- makeResolvSeed defaultResolvConf
---   >>> withResolver rs $ \resolver -> lookupMX resolver hostname
+--   >>> withResolver rs $ \resolver -> lookupMX resolver "mew.org"
 --   Right [("mail.mew.org.",10)]
 --
 --   Also note that all hostnames are returned with a trailing dot to
@@ -167,7 +164,7 @@ lookupMX rlv dom = do
     Right rds -> return $ mapM unTag rds
   where
     unTag :: RData -> Either DNSError (Domain,Int)
-    unTag (RD_MX pr dm) = Right (dm,pr)
+    unTag (RD_MX pr dm) = Right (dm, fromIntegral pr)
     unTag _ = Left UnexpectedRDATA
 
 -- | Look up all \'MX\' records for the given hostname, and then
@@ -177,10 +174,8 @@ lookupMX rlv dom = do
 --   Examples:
 --
 --   >>> import Data.List (sort)
---   >>> let hostname = Data.ByteString.Char8.pack "wide.ad.jp"
---   >>>
 --   >>> rs <- makeResolvSeed defaultResolvConf
---   >>> ips <- withResolver rs $ \resolver -> lookupAviaMX resolver hostname
+--   >>> ips <- withResolver rs $ \resolver -> lookupAviaMX resolver "wide.ad.jp"
 --   >>> fmap sort ips
 --   Right [133.138.10.39,203.178.136.30]
 --
@@ -253,10 +248,8 @@ lookupNSImpl lookup_function rlv dom = do
 --   Examples:
 --
 --   >>> import Data.List (sort)
---   >>> let hostname = Data.ByteString.Char8.pack "mew.org"
---   >>>
 --   >>> rs <- makeResolvSeed defaultResolvConf
---   >>> ns <- withResolver rs $ \resolver -> lookupNS resolver hostname
+--   >>> ns <- withResolver rs $ \resolver -> lookupNS resolver "mew.org"
 --   >>> fmap sort ns
 --   Right ["ns1.mew.org.","ns2.mew.org."]
 --
@@ -277,12 +270,10 @@ lookupNS = lookupNSImpl DNS.lookup
 --   the IP address of which was found beforehand:
 --
 --   >>> import Data.List (sort)
---   >>> let hostname = Data.ByteString.Char8.pack "example.com"
---   >>>
 --   >>> let ri = RCHostName "192.5.6.30" -- a.gtld-servers.net
 --   >>> let rc = defaultResolvConf { resolvInfo = ri }
 --   >>> rs <- makeResolvSeed rc
---   >>> ns <- withResolver rs $ \resolver -> lookupNSAuth resolver hostname
+--   >>> ns <- withResolver rs $ \resolver -> lookupNSAuth resolver "example.com"
 --   >>> fmap sort ns
 --   Right ["a.iana-servers.net.","b.iana-servers.net."]
 --
@@ -300,10 +291,8 @@ lookupNSAuth = lookupNSImpl DNS.lookupAuth
 --   <http://en.wikipedia.org/wiki/DomainKeys_Identified_Mail>. As an
 --   example, we find the SPF record for \"mew.org\":
 --
---   >>> let hostname = Data.ByteString.Char8.pack "mew.org"
---   >>>
 --   >>> rs <- makeResolvSeed defaultResolvConf
---   >>> withResolver rs $ \resolver -> lookupTXT resolver hostname
+--   >>> withResolver rs $ \resolver -> lookupTXT resolver "mew.org"
 --   Right ["v=spf1 +mx -all"]
 --
 lookupTXT :: Resolver -> Domain -> IO (Either DNSError [ByteString])
@@ -320,6 +309,31 @@ lookupTXT rlv dom = do
 
 ----------------------------------------------------------------
 
+-- | Look up the \'SOA\' record for the given domain. The results include the
+--   domain, mailbox, serial number, refresh time, retry time, expiration
+--   time, and minimum TTL.
+--
+--   This can be useful to validate TTLs for a server or get an abuse
+--   contact address for a domain.
+--
+--   >>> rs <- makeResolvSeed defaultResolvConf
+--   >>> withResolver rs $ \resolver -> lookupSOA resolver "mew.org"
+--   Right [("ns1.mew.org.","kazu@mew.org.",201406240,3600,300,3600000,3600)]
+--
+lookupSOA :: Resolver -> Domain -> IO (Either DNSError [(Domain,Mailbox,Word32,Word32,Word32,Word32,Word32)])
+lookupSOA rlv dom = do
+  erds <- DNS.lookup rlv dom SOA
+  case erds of
+    -- See lookupXviaMX for an explanation of this construct.
+    Left err  -> return (Left err)
+    Right rds -> return $ mapM unTag rds
+  where
+    unTag :: RData -> Either DNSError (Domain,Mailbox,Word32,Word32,Word32,Word32,Word32)
+    unTag (RD_SOA mn mr serial refresh retry expire mini) = Right (mn, mr, serial, refresh, retry, expire, mini)
+    unTag _ = Left UnexpectedRDATA
+
+----------------------------------------------------------------
+
 -- | Look up all \'PTR\' records for the given hostname. To perform a
 --   reverse lookup on an IP address, you must first reverse its
 --   octets and then append the suffix \".in-addr.arpa.\"
@@ -327,10 +341,8 @@ lookupTXT rlv dom = do
 --   We look up the PTR associated with the IP address
 --   210.130.137.80, i.e., 80.137.130.210.in-addr.arpa:
 --
---   >>> let hostname = Data.ByteString.Char8.pack "164.2.232.202.in-addr.arpa"
---   >>>
 --   >>> rs <- makeResolvSeed defaultResolvConf
---   >>> withResolver rs $ \resolver -> lookupPTR resolver hostname
+--   >>> withResolver rs $ \resolver -> lookupPTR resolver "164.2.232.202.in-addr.arpa"
 --   Right ["www.iij.ad.jp."]
 --
 --   The 'lookupRDNS' function is more suited to this particular task.
@@ -354,10 +366,8 @@ lookupPTR rlv dom = do
 --   We repeat the example from 'lookupPTR', except now we pass the IP
 --   address directly:
 --
---   >>> let hostname = Data.ByteString.Char8.pack "202.232.2.164"
---   >>>
 --   >>> rs <- makeResolvSeed defaultResolvConf
---   >>> withResolver rs $ \resolver -> lookupRDNS resolver hostname
+--   >>> withResolver rs $ \resolver -> lookupRDNS resolver "202.232.2.164"
 --   Right ["www.iij.ad.jp."]
 --
 lookupRDNS :: Resolver -> Domain -> IO (Either DNSError [Domain])
@@ -392,10 +402,8 @@ lookupRDNS rlv ip = lookupPTR rlv dom
 --
 --   Examples:
 --
---   >>> let q = Data.ByteString.Char8.pack "_xmpp-server._tcp.jabber.ietf.org"
---   >>>
 --   >>> rs <- makeResolvSeed defaultResolvConf
---   >>> withResolver rs $ \resolver -> lookupSRV resolver q
+--   >>> withResolver rs $ \resolver -> lookupSRV resolver "_xmpp-server._tcp.jabber.ietf.org"
 --   Right [(5,0,5269,"jabber.ietf.org.")]
 
 -- Though the "jabber.ietf.orgs" SRV record may prove reasonably stable, as
@@ -404,7 +412,7 @@ lookupRDNS rlv ip = lookupPTR rlv dom
 -- example should be displayed in a format that is not recognized as a test
 -- by "doctest".
 
-lookupSRV :: Resolver -> Domain -> IO (Either DNSError [(Int,Int,Int,Domain)])
+lookupSRV :: Resolver -> Domain -> IO (Either DNSError [(Word16, Word16, Word16, Domain)])
 lookupSRV rlv dom = do
   erds <- DNS.lookup rlv dom SRV
   case erds of
@@ -412,6 +420,6 @@ lookupSRV rlv dom = do
     Left err  -> return (Left err)
     Right rds -> return $ mapM unTag rds
   where
-    unTag :: RData -> Either DNSError (Int,Int,Int,Domain)
+    unTag :: RData -> Either DNSError (Word16, Word16, Word16, Domain)
     unTag (RD_SRV pri wei prt dm) = Right (pri,wei,prt,dm)
     unTag _ = Left UnexpectedRDATA
