@@ -1,30 +1,54 @@
-{-# LANGUAGE TypeSynonymInstances, FlexibleInstances, CPP #-}
-module Network.DNS.StateBinary where
+{-# LANGUAGE TypeSynonymInstances #-}
+{-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE CPP #-}
+module Network.DNS.StateBinary (
+    PState(..)
+  , initialState
+  , SPut
+  , runSPut
+  , put8
+  , put16
+  , put32
+  , putInt8
+  , putInt16
+  , putInt32
+  , putByteString
+  , SGet
+  , runSGet
+  , runSGetWithLeftovers
+  , get8
+  , get16
+  , get32
+  , getInt8
+  , getInt16
+  , getInt32
+  , getNByteString
+  , getPosition
+  , wsPop
+  , wsPush
+  , wsPosition
+  , addPositionW
+  , push
+  , pop
+  , getNBytes
+  ) where
 
 import Control.Monad.State (State, StateT)
 import qualified Control.Monad.State as ST
-import Control.Monad.Trans.Resource (ResourceT)
 import qualified Data.Attoparsec.ByteString as A
-import qualified Data.Attoparsec.ByteString.Lazy as AL
 import qualified Data.Attoparsec.Types as T
-import Data.ByteString (ByteString)
 import qualified Data.ByteString as BS
 import Data.ByteString.Builder (Builder)
 import qualified Data.ByteString.Builder as BB
-import qualified Data.ByteString.Lazy as BL
-import Data.Conduit (Sink)
-import Data.Conduit.Attoparsec (sinkParser)
+import qualified Data.ByteString.Lazy.Char8 as LBS
 import Data.IntMap (IntMap)
 import qualified Data.IntMap as IM
 import Data.Map (Map)
 import qualified Data.Map as M
-import Data.Word (Word8, Word16, Word32)
-import Network.DNS.Types
+import Data.Semigroup as Sem
 
-#if __GLASGOW_HASKELL__ < 709
-import Control.Applicative ((<$>), (<*))
-import Data.Monoid (Monoid, mconcat, mappend, mempty)
-#endif
+import Network.DNS.Imports
+import Network.DNS.Types
 
 ----------------------------------------------------------------
 
@@ -38,9 +62,14 @@ data WState = WState {
 initialWState :: WState
 initialWState = WState M.empty 0
 
+instance Sem.Semigroup SPut where
+    p1 <> p2 = (Sem.<>) <$> p1 <*> p2
+
 instance Monoid SPut where
     mempty = return mempty
-    mappend a b = mconcat <$> sequence [a, b]
+#if !(MIN_VERSION_base(4,11,0))
+    mappend = (Sem.<>)
+#endif
 
 put8 :: Word8 -> SPut
 put8 = fixedSized 1 BB.word8
@@ -162,18 +191,16 @@ getNByteString n = ST.lift (A.take n) <* addPosition n
 initialState :: PState
 initialState = PState IM.empty 0
 
-sinkSGet :: SGet a -> Sink ByteString (ResourceT IO) (a, PState)
-sinkSGet parser = sinkParser (ST.runStateT parser initialState)
+runSGet :: SGet a -> ByteString -> Either String (a, PState)
+runSGet parser bs = A.eitherResult $ A.parse (ST.runStateT parser initialState) bs
 
-runSGet :: SGet a -> BL.ByteString -> Either String (a, PState)
-runSGet parser bs = AL.eitherResult $ AL.parse (ST.runStateT parser initialState) bs
-
-runSGetWithLeftovers :: SGet a -> BL.ByteString -> Either String ((a, PState), BL.ByteString)
-runSGetWithLeftovers parser bs = toResult $ AL.parse (ST.runStateT parser initialState) bs
+runSGetWithLeftovers :: SGet a -> ByteString -> Either String ((a, PState), ByteString)
+runSGetWithLeftovers parser bs = toResult $ A.parse (ST.runStateT parser initialState) bs
   where
-    toResult :: AL.Result r -> Either String (r, BL.ByteString)
-    toResult (AL.Done i r) = Right (r, i)
-    toResult (AL.Fail _ _ err) = Left err
+    toResult :: A.Result r -> Either String (r, ByteString)
+    toResult (A.Done i r) = Right (r, i)
+    toResult (A.Partial f) = toResult $ f BS.empty
+    toResult (A.Fail _ _ err) = Left err
 
-runSPut :: SPut -> BL.ByteString
-runSPut = BB.toLazyByteString . flip ST.evalState initialWState
+runSPut :: SPut -> ByteString
+runSPut = LBS.toStrict . BB.toLazyByteString . flip ST.evalState initialWState
